from django.core.management.base import BaseCommand, CommandError

from storelocator.updaters import yahoo


class Command(BaseCommand):

    help = (
        "Update existing locations with latitude and longitude chosen service")

    def handle(self, *args, **kwargs):
        try:
            time_unit = args[0]
        except IndexError:
            raise CommandError('Choose a timedelta time_unit.')

        try:
            value = int(args[1])
        except IndexError:
            raise CommandError('You must provide a time value.')

        try:
            service = args[2]
        except IndexError:
            raise CommandError('You must provide a service name.')

        if service == "yahoo":
            yahoo.yql_update_locations(time_unit, value)
        else:
            raise CommandError('You must provide a valid service name.')



from .models import Shop

import logging
import requests
import time


logger = logging.getLogger('storelocator')


def update_shops():
    limit = 2500
    for shop in Shop.objects.filter(latitude=None, longitude=None)[:limit]:
        location = "%s %s %s" % (shop.city, shop.postalcode, shop.street)
        try:
            json = reverse_geocoding(location)
        except requests.HTTPError:
            continue

        geo = json['results'][0]['geometry']['location']
        shop.latitude = geo['lat']
        shop.longitude = geo['lng']
        shop.save()
        logger.debug('Saved lat & lon for: %s' % shop)


def reverse_geocoding(location):
    url = "http://maps.googleapis.com/maps/api/geocode/json"
    qs = "?address=%s&components=country:Germany&sensor=false" % location
    combined = url+qs
    attempts = 0
    success = False
    max_attempts = 3

    while success != True and attempts < max_attempts:
        response = requests.get(combined).json()
        attempts += 1
        status = response.get('status')

        if status == "OVER_QUERY_LIMIT":
            logger.debug('API Limit reached. Sleeping 2 seconds')
            time.sleep(2)
            continue

        if status == "ZERO_RESULTS":
            logger.debug("Zero results: %s" % location)
            raise requests.HTTPError()
        success = True
        return response

    if attempts == max_attempts:
        logger.debug("Can't fetch geocoding for: %s" % location)
        raise requests.HTTPError()



from django.conf import settings
from django.utils import timezone

from storelocator.models import Location, Shop

from datetime import timedelta
import logging
import yql


logger = logging.getLogger('storelocator')


def yql_request(query):
    """
    Update locations older than x days using YQL.
    """
    API_KEY = getattr(settings, "YQL_API_KEY", None)
    SECRET = getattr(settings, "YQL_CONSUMER_SECRET", None)
    if not API_KEY and not SECRET:
        raise Exception('No api key and secret found.')

    y = yql.TwoLegged(API_KEY, SECRET)

    try:
        result = y.execute(query)
    except yql.YQLError as e:
        raise e

    logger.debug("%s" % query)
    return result


def update_qs_locations(qs, fields, last_updated=None):
    for obj in qs:
        filt = ' '.join([getattr(obj, attr).encode('utf-8') for attr in fields])
        query = "select centroid from geo.places where text='%s'" % filt

        count = 0
        tries = 4

        while True:
            try:
                result = yql_request(query)
            except yql.YQLError as e:
                logger.error(e)
                count += 1
                if count >= tries:
                    raise e
                continue
            break

        try:
            coordinates = result.one()['centroid']
        except IndexError:
            logger.error("Can't fetch query: %s" % filt)
            continue
        except yql.NotOneError:
            coordinates = result.results['place'][0]['centroid']

        new_lat = float(coordinates['latitude'])
        new_lon = float(coordinates['longitude'])

        obj.latitude = new_lat
        obj.longitude = new_lon

        if last_updated:
            obj.last_updated = last_updated
        obj.save()


def yql_update_locations(time_unit, value):
    """
    Use YQL to update locations older than <time unit> <value>.
    """
    now = timezone.now()
    since = now - timedelta(**{time_unit: value})
    locations = Location.objects.filter(last_updated__lte=since)
    update_qs_locations(locations, ['state', 'city', 'postalcode'], now)


def yql_update_shops():
    """
    Use YQL to update shops where longitude and latitude are empty.
    """
    shops = Shop.objects.filter(latitude=None, longitude=None)
    update_qs_locations(shops, ['city', 'postalcode', 'street'])



from django.db.models.query import QuerySet
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.views.decorators.cache import cache_page
from django.views.generic.base import View
from django.views.generic.edit import FormMixin
from django.utils.decorators import method_decorator
from django.utils import translation

from .forms import StoreLocatorForm
from .models import Location, StoreLocator
from .utils import get_near_shops

import json


class ExtendedEncoder(json.JSONEncoder):

    def default(self, obj):
        if hasattr(obj, 'serialize'):
            return obj.serialize()
        elif isinstance(obj, QuerySet):
            return list(obj)
        return json.JSONEncoder.default(self, obj)


class StoreLocatorView(FormMixin, View):
    """
    A view for processing a form with json response
    """

    form_class = StoreLocatorForm

    @method_decorator(cache_page(60 * 60))
    def dispatch(self, request, slug, *args, **kwargs):
        self.storelocator = get_object_or_404(StoreLocator, slug=slug)
        self.callback = request.GET.get('callback')
        self.language = request.GET.get('language')

        if self.language:
            translation.activate(self.language)
            request.LANGUAGE_CODE = translation.get_language()

        return super(StoreLocatorView, self).dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)

    def get_form_kwargs(self):
        return {'data': self.request.GET}

    def form_valid(self, form):
        result_limit = form.limit_max
        filter_radius = form.radius_max

        context = self.get_json_context(form)

        latitude = form.cleaned_data.get('latitude')
        longitude = form.cleaned_data.get('longitude')
        postalcode = form.cleaned_data.get('postalcode')
        iso = form.cleaned_data.get('iso')
        limit = form.cleaned_data.get('limit')
        radius = form.cleaned_data.get('radius')

        if limit == 0:
            result_limit = limit

        if limit:
            result_limit = limit

        if radius:
            filter_radius = radius

        if latitude and longitude:
            context['shops'] = get_near_shops(
                self.storelocator,
                latitude,
                longitude,
                result_limit,
                filter_radius,
                True,
                iso=None)

        if postalcode:
            locations = Location.objects.filter(postalcode=postalcode)

            if locations:
                location = locations[0]
                context['shops'] = get_near_shops(
                    self.storelocator,
                    location.latitude,
                    location.longitude,
                    result_limit,
                    filter_radius,
                    False,
                    iso)

        return self.json_response(context)

    def form_invalid(self, form):
        context = self.get_json_context(form)
        return self.json_response(context)

    def get_json_context(self, form):
        errors = form.errors.copy()
        if errors.get('__all__'):
            del errors['__all__']

        context = {
            'success': form.is_valid(),
            'form_errors': errors,
            'non_field_errors': form.non_field_errors(),
            'shops': [],
        }
        return context

    def json_response(self, context):
        serialized = json.dumps(context, cls=ExtendedEncoder)

        if self.callback:
            serialized = "%s(%s)" % (self.callback, serialized)

        return HttpResponse(serialized, mimetype='application/json')



from django.contrib import admin
from django.core.urlresolvers import reverse

from .models import Location, StoreLocator, Shop


class LocationAdmin(admin.ModelAdmin):
    list_display = (
        'postalcode', 'iso', 'city', 'state', 'latitude', 'longitude',
        'last_updated'
    )

    ordering = ['postalcode',]

    list_filter = [
        'iso',
    ]

    search_fields = ('state', 'city', 'iso', 'postalcode')


class StoreLocatorAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'slug'
    )

    search_fields = ('name', 'slug')


class ShopAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'type', 'street', 'postalcode', 'city',
        'iso', 'latitude', 'longitude', 'get_storelocator_name'
    )
    search_fields = ('name', 'city', 'postalcode', 'street')
    list_filter = ('type', 'storelocator__name')
    raw_id_fields = ('storelocator', )

    def get_storelocator_name(self, obj):
        url = reverse('admin:%s_%s_change' % (
            obj.storelocator._meta.app_label,
            obj.storelocator._meta.module_name
        ), args=[obj.storelocator.id] )
        return u'<a href="%s">%s</a>' % (url,  obj.storelocator.name)
    get_storelocator_name.short_description = 'storelocator'
    get_storelocator_name.allow_tags = True


admin.site.register(Location, LocationAdmin)
admin.site.register(StoreLocator, StoreLocatorAdmin)
admin.site.register(Shop, ShopAdmin)


from .importer import UnicodeCSVImporter
from storelocator.models import Location


class GeonamesLocationImporter(UnicodeCSVImporter):

    """
    Imports locations from http://download.geonames.org/export/zip/
    """

    def run(self):
        bulk_objects = []

        for row in self.rows:
            bulk_objects.append(
                Location(
                    iso=row[0],
                    postalcode=row[1],
                    city=row[2],
                    state=row[3],
                    latitude=row[9],
                    longitude=row[10]))

        Location.objects.bulk_create(bulk_objects)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Location'
        db.create_table(u'storelocator_location', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('iso', self.gf('django.db.models.fields.CharField')(max_length=2)),
            ('postalcode', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('city', self.gf('django.db.models.fields.CharField')(max_length=180)),
            ('state', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('latitude', self.gf('django.db.models.fields.FloatField')()),
            ('longitude', self.gf('django.db.models.fields.FloatField')()),
            ('last_updated', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'storelocator', ['Location'])

        # Adding model 'StoreLocator'
        db.create_table(u'storelocator_storelocator', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('slug', self.gf('django.db.models.fields.CharField')(max_length=55)),
        ))
        db.send_create_signal(u'storelocator', ['StoreLocator'])

        # Adding model 'Shop'
        db.create_table(u'storelocator_shop', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=140)),
            ('city', self.gf('django.db.models.fields.CharField')(max_length=180)),
            ('postalcode', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('street', self.gf('django.db.models.fields.CharField')(max_length=140)),
            ('iso', self.gf('django.db.models.fields.CharField')(max_length=2)),
            ('latitude', self.gf('django.db.models.fields.FloatField')(null=True, blank=True)),
            ('longitude', self.gf('django.db.models.fields.FloatField')(null=True, blank=True)),
            ('storelocator', self.gf('django.db.models.fields.related.ForeignKey')(related_name='shops', to=orm['storelocator.StoreLocator'])),
        ))
        db.send_create_signal(u'storelocator', ['Shop'])


    def backwards(self, orm):
        # Deleting model 'Location'
        db.delete_table(u'storelocator_location')

        # Deleting model 'StoreLocator'
        db.delete_table(u'storelocator_storelocator')

        # Deleting model 'Shop'
        db.delete_table(u'storelocator_shop')


    models = {
        u'storelocator.location': {
            'Meta': {'ordering': "['iso', 'postalcode']", 'object_name': 'Location'},
            'city': ('django.db.models.fields.CharField', [], {'max_length': '180'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'iso': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'last_updated': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'latitude': ('django.db.models.fields.FloatField', [], {}),
            'longitude': ('django.db.models.fields.FloatField', [], {}),
            'postalcode': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'})
        },
        u'storelocator.shop': {
            'Meta': {'object_name': 'Shop'},
            'city': ('django.db.models.fields.CharField', [], {'max_length': '180'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'iso': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'latitude': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'longitude': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '140'}),
            'postalcode': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'storelocator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'shops'", 'to': u"orm['storelocator.StoreLocator']"}),
            'street': ('django.db.models.fields.CharField', [], {'max_length': '140'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        u'storelocator.storelocator': {
            'Meta': {'object_name': 'StoreLocator'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'slug': ('django.db.models.fields.CharField', [], {'max_length': '55'})
        }
    }

    complete_apps = ['storelocator']


from django.core.management.base import BaseCommand, CommandError

from storelocator.importers import shops


class Command(BaseCommand):
    help = 'Imports shops from csv'

    def handle(self, *args, **kwargs):
        try:
            path = args[0]
        except IndexError:
            raise CommandError('You must provide a path for your csv file.')

        try:
            storelocator_id = args[1]
        except IndexError:
            raise CommandError('You must provide the id of a storelocator obj')

        importer = shops.ShopImporter(path, dialect='excel-tab', delimiter=',')
        importer.run(storelocator_id)



from django import forms
from django.conf import settings
from django.core.validators import MinValueValidator, MaxValueValidator
from django.utils.translation import ugettext_lazy as _


class StoreLocatorForm(forms.Form):

    limit_max = getattr(settings, 'STORELOCATOR_LIMIT_MAX', 15)
    radius_max = getattr(settings, 'STORELOCATOR_RADIUS_MAX', 10.0)

    postalcode = forms.CharField(max_length=20, required=False)
    latitude = forms.FloatField(required=False)
    longitude = forms.FloatField(required=False)
    iso = forms.CharField(max_length=2, required=False)
    limit = forms.IntegerField(required=False, validators=[
        MinValueValidator(0), MaxValueValidator(limit_max)])
    radius = forms.FloatField(required=False, validators=[
        MinValueValidator(0.1), MaxValueValidator(radius_max)])

    def clean(self):
        cleaned_data = super(StoreLocatorForm, self).clean()

        latitude = cleaned_data.get('latitude')
        longitude = cleaned_data.get('longitude')
        postalcode = cleaned_data.get('postalcode')
        iso = cleaned_data.get('iso')

        # check if all fields are empty
        if not latitude and not longitude and not postalcode:
            if (not 'latitude' in self._errors and
                not 'longitude' in self._errors and
                not 'postalcode' in self._errors):
                raise forms.ValidationError(
                    _("'postalcode' or 'latitude' and "
                      "'longitude' are required."))

        if latitude and longitude:
            return cleaned_data

        if postalcode:
            if not iso:
                raise forms.ValidationError(
                    _("'iso' is required."))

        return cleaned_data



from django.test import TestCase

from storelocator.importers import locations, shops
from storelocator.factories import StoreLocatorFactory
from storelocator.models import Location, Shop

import os


class LocationImportTest(TestCase):

    def setUp(self):
        self.csv = os.path.abspath(os.path.join(
            os.path.dirname(__file__), 'testdata', 'testlocations.csv'))
        self.importer = locations.GeonamesLocationImporter(
            self.csv, dialect='excel-tab')

    def test_import_locations(self):
        self.importer.run()
        locations = Location.objects.all()
        self.assertEqual(3, locations.count())
        self.assertEqual(2, Location.objects.filter(iso='DE').count())
        self.assertEqual(1, Location.objects.filter(iso='RU').count())


class ShopImportTest(TestCase):

    def setUp(self):
        self.storelocator = StoreLocatorFactory.create()
        self.csv = os.path.abspath(os.path.join(
            os.path.dirname(__file__), 'testdata', 'testshops.csv'))
        self.importer = shops.ShopImporter(
            self.csv, dialect='excel-tab', delimiter=',')

    def test_import_locations(self):
        self.importer.run(self.storelocator.id)
        shops = Shop.objects.all()
        self.assertEqual(4, shops.count())



from django.test import TestCase

from storelocator.factories import (
    ShopFactory, LocationFactory, StoreLocatorFactory)

from geopy import distance
import random


def prepare_shops(testcase):
    testcase.user_location = (52.5565, 13.3911)
    testcase.shop_locations = [
        (52.5655, 13.4001),  # 1.17 km
        (52.5745, 13.4091),  # 2.35 km
        (52.5835, 13.4181),  # 3.52 km
        (52.5925, 13.4271),  # 4.69 km
        (52.6015, 13.4361),  # 5.86 km
        (52.6105, 13.4451),  # 7.04 km
        (52.6195, 13.4541),  # 8.21 km
        (52.6285, 13.4631),  # 9.38 km
        (52.6375, 13.4721),  # 10.55 km
        (52.6465, 13.4811),  # 11.73 km
        (52.6555, 13.4901),  # 12.90 km
    ]
    random.shuffle(testcase.shop_locations)

    testcase.userpos = LocationFactory.create(
        latitude=52.5565, longitude=13.3911, postalcode='13359')

    testcase.shops = []
    testcase.storelocator = StoreLocatorFactory.create(
        name="Test Locator", slug="testlocator")
    for i, loc in enumerate(testcase.shop_locations):
        lat, lon = loc
        shop = ShopFactory.create(
            latitude=lat, longitude=lon, storelocator=testcase.storelocator)
        shop.distance = distance.distance(testcase.user_location, loc)
        testcase.shops.append(shop)


class LocationTestCase(TestCase):

    def setUp(self):
        prepare_shops(self)



from django.core.management.base import BaseCommand, CommandError

from storelocator.importers import locations


class Command(BaseCommand):
    help = 'Imports locations from geonames.org'

    def handle(self, *args, **kwargs):
        try:
            path = args[0]
        except IndexError:
            raise CommandError('You must provide a path for your csv file.')

        importer = locations.GeonamesLocationImporter(path, dialect='excel-tab')
        importer.run()



import csv


def unicode_csv_reader(file, dialect, **kwargs):
    csv_reader = csv.reader(file, dialect=dialect, **kwargs)
    for row in csv_reader:
        yield [unicode(cell, 'utf-8') for cell in row]


class UnicodeCSVImporter(object):

    """
    Base csv importer class. Reads unicode rows into self.rows
    """

    def __init__(self, file, **kwargs):
        self.rows = [row for row in unicode_csv_reader(
            open(file, 'r'), **kwargs)]

    def run(self):
        raise NotImplementedError



from .importer import UnicodeCSVImporter
from storelocator.models import Shop, StoreLocator


class ShopImporter(UnicodeCSVImporter):

    """
    Imports shops in a custom format.
    First row needs the following fields:
        type, name, street, postalcode, city, state, iso
    """

    def run(self, storelocator_id):
        try:
            storelocator = StoreLocator.objects.get(id=storelocator_id)
        except StoreLocator.DoesNotExist:
            raise Exception(
                "Can't find StoreLocator object with id: %i" % storelocator_id)

        data_map = {
            "type": 0,
            "name": 1,
            "street": 2,
            "postalcode": 3,
            "city": 4,
            "state": 5,
            "iso": 6
        }

        # get rid of double entries
        rows = set(map(tuple, self.rows))

        # save shops
        bulk_objects = []
        for m in rows:
            bulk_objects.append(
                Shop(
                    type = m[data_map["type"]],
                    name = m[data_map["name"]],
                    city = m[data_map["city"]],
                    postalcode = m[data_map["postalcode"]],
                    street = m[data_map["street"]],
                    iso = m[data_map["iso"]],
                    storelocator_id = storelocator.id))

        Shop.objects.bulk_create(bulk_objects)


from .models import Shop


def get_near_shops(storelocator, lat, lon, limit, radius, distance, iso=None):

    shops = storelocator.shops.filter(
        latitude__gte=(lat - 0.009 * radius),
        latitude__lte=(lat + 0.009 * radius),
        longitude__gte=(lon - 0.009 * radius),
        longitude__lte=(lon + 0.009 * radius),
    )

    if iso:
        shops = shops.filter(iso=iso)

    if distance:
        for shop in list(shops):
            shop.distance = shop.distance_to(lat, lon).km
        shops = [shop for shop in shops if shop.distance <= radius]
        shops = sorted(shops, key=lambda s: s.distance)
    return shops[:limit]




from django.core.management.base import BaseCommand, CommandError

from storelocator.updaters import google
from storelocator.updaters import yahoo


class Command(BaseCommand):
    help = 'Update shops latitude and longitude with chosen service'

    def handle(self, *args, **kwargs):
        try:
            service = args[0]
        except IndexError:
            raise CommandError('You must provide a service name.')

        if service == "google":
            google.update_shops()
        if service == "yahoo":
            yahoo.yql_update_shops()
        else:
            raise CommandError('You must provide a valid service name.')



from django.test import TestCase

from storelocator.forms import StoreLocatorForm


class StoreLocatorFormTest(TestCase):

    def test_form_empty_invalid(self):
        form = StoreLocatorForm({})
        valid = form.is_valid()
        self.assertEqual(1, len(form.errors))
        self.assertTrue(form.non_field_errors())
        self.assertFalse(valid)

    def test_form_valid_latlon(self):
        form = StoreLocatorForm({'latitude': 48.0, 'longitude': 6.0})
        valid = form.is_valid()
        self.assertTrue(valid)

    def test_form_valid_postalcode(self):
        form = StoreLocatorForm({'postalcode': '13359', 'iso': 'DE'})
        valid = form.is_valid()
        self.assertTrue(valid)

    def test_form_invalid_postalcode(self):
        form = StoreLocatorForm({'postalcode': '13359'})
        valid = form.is_valid()
        self.assertFalse(valid)

    def test_limit_validator_valid(self):
        form = StoreLocatorForm({
            'postalcode': '13359',
            'iso': 'DE',
            'limit': 1
        })
        valid = form.is_valid()
        self.assertTrue(valid)

    def test_limit_validator_invalid_min(self):
        form = StoreLocatorForm({
            'postalcode': '13359',
            'iso': 'DE',
            'limit': -1
        })
        valid = form.is_valid()
        self.assertFalse(valid)
        self.assertIn('limit', form.errors)

    def test_limit_validator_invalid_max(self):
        form = StoreLocatorForm({
            'postalcode': '13359',
            'iso': 'DE',
            'limit': 16
        })
        valid = form.is_valid()
        self.assertFalse(valid)
        self.assertIn('limit', form.errors)

    def test_radius_validator_valid(self):
        form = StoreLocatorForm({
            'postalcode': '13359',
            'iso': 'DE',
            'radius': 8.0
        })
        valid = form.is_valid()
        self.assertTrue(valid)

    def test_radius_validator_invalid_min(self):
        form = StoreLocatorForm({
            'postalcode': '13359',
            'iso': 'DE',
            'radius': -1
        })
        valid = form.is_valid()
        self.assertFalse(valid)
        self.assertIn('radius', form.errors)

    def test_radius_validator_invalid_max(self):
        form = StoreLocatorForm({
            'postalcode': '13359',
            'iso': 'DE',
            'radius': 11.0
        })
        valid = form.is_valid()
        self.assertFalse(valid)
        self.assertIn('radius', form.errors)



from .testcase import LocationTestCase
from storelocator.factories import StoreLocatorFactory
from storelocator.utils import get_near_shops


class GetNearShopsTest(LocationTestCase):

    def test_get_near_shops(self):
        shops = get_near_shops(
            self.storelocator,
            self.userpos.latitude,
            self.userpos.longitude,
            5,
            10.0,
            True)

        self.assertEqual(5, len(shops))
        for shop in shops:
            self.assertTrue(shop.distance <= 5.9)  # 5th shop has dist 5.86

    def test_smaller_range(self):
        shops = get_near_shops(
            self.storelocator,
            self.userpos.latitude,
            self.userpos.longitude,
            5,
            5,
            True)

        self.assertEqual(4, len(shops))
        for shop in shops:
            self.assertTrue(shop.distance <= 4.7)  # 4th shop has dist 4.69

    def test_range(self):
        shops = get_near_shops(
            self.storelocator,
            self.userpos.latitude,
            self.userpos.longitude,
            15,
            13.0,
            True)

        self.assertEqual(11, len(shops))
        for shop in shops:
            self.assertTrue(shop.distance <= 13.0)  # last shop has dist 12.90

    def test_other_storelocator(self):
        storelocator2 = StoreLocatorFactory.create(
            name="Store Locator 2", slug="store-locator-2")

        shops = get_near_shops(
            storelocator2,
            self.userpos.latitude,
            self.userpos.longitude,
            15,
            10.0,
            True)

        self.assertEqual(0, len(shops))

    def test_iso_filtering(self):
        shops = get_near_shops(
            self.storelocator,
            self.userpos.latitude,
            self.userpos.longitude,
            15,
            10.0,
            True,
            'NL')
        self.assertEqual(0, len(shops))

    def test_no_distance_property(self):
        shops = get_near_shops(
            self.storelocator,
            self.userpos.latitude,
            self.userpos.longitude,
            5,
            1.17,
            False)

        self.assertEqual(1, len(shops))
        self.assertFalse(hasattr(shops[0], "distance"))

    def test_distance_filter(self):
        shops = get_near_shops(
            self.storelocator,
            self.userpos.latitude,
            self.userpos.longitude,
            5,
            2.35,
            True)

        self.assertEqual(2, len(shops))

    def test_limit_one(self):
        shops = get_near_shops(
            self.storelocator,
            self.userpos.latitude,
            self.userpos.longitude,
            1,
            10.0,
            True)

        self.assertEqual(1, len(shops))

    def test_limit_zero(self):
        shops = get_near_shops(
            self.storelocator,
            self.userpos.latitude,
            self.userpos.longitude,
            0,
            10.0,
            True)

        self.assertEqual(0, len(shops))



import os
from setuptools import setup, find_packages

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-storelocator',
    version='0.1',
    packages=find_packages(),
    zip_safe=False,
    license='MIT',
    description='Django Storelocator is a Django App for locating stores near a geographical location.',
    long_description=open('README.rst').read(),
    url='https://github.com/moccu',
    author='Carlo Smouter',
    author_email='info@moccu.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    dependency_links=['https://github.com/lockwooddev/python-yql'],
    install_requires=[
        "geopy >= 0.95.1",
        "requests >= 1.1.0",
    ],
)



import os


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'testrunner',
        'KEY_PREFIX': 'testrunner',
    }
}

INSTALLED_APPS = [
    'storelocator',
]

LOGGING_CONFIG = None

SECRET_KEY = "blup"

TEST_RUNNER = 'discover_runner.DiscoverRunner'

ROOT_URLCONF = 'storelocator.urls'

USE_TZ = True



from django.core.cache import cache
from django.core.urlresolvers import reverse

from .testcase import LocationTestCase

import json
import urllib


class LocationFormViewTest(LocationTestCase):

    def test_404_response(self):
        response = self.client.get(
            reverse('storelocator', args=['invalid']))
        self.assertEqual(404, response.status_code)

    def test_405_response(self):
        response = self.client.post(
            reverse('storelocator', args=['testlocator']))
        self.assertEqual(405, response.status_code)

    def test_nearby_shops_postalcode(self):
        form_data = {'postalcode': '13359', 'iso': 'DE'}
        encoded = urllib.urlencode(form_data)
        response = self.client.get(
            reverse('storelocator', args=['testlocator'])+'?'+encoded)
        json_response = json.loads(response.content)
        self.assertEqual(True, json_response['success'])
        self.assertEqual(200, response.status_code)
        self.assertTrue(json_response['shops'])
        self.assertFalse(json_response['non_field_errors'])
        self.assertFalse(json_response['form_errors'])

    def test_no_nearby_shops_postalcode(self):
        form_data = {'postalcode': '12345', 'iso': 'DE'}
        encoded = urllib.urlencode(form_data)
        response = self.client.get(
            reverse('storelocator', args=['testlocator'])+'?'+encoded)
        json_response = json.loads(response.content)
        self.assertEqual(True, json_response['success'])
        self.assertEqual(200, response.status_code)
        self.assertEqual(0, len(json_response['shops']))
        self.assertFalse(json_response['non_field_errors'])
        self.assertFalse(json_response['form_errors'])

    def test_postalcode_no_iso(self):
        form_data = {'postalcode': '12345'}
        encoded = urllib.urlencode(form_data)
        response = self.client.get(
            reverse('storelocator', args=['testlocator'])+'?'+encoded)
        json_response = json.loads(response.content)
        self.assertEqual(False, json_response['success'])
        self.assertEqual(200, response.status_code)
        self.assertEqual(0, len(json_response['shops']))
        self.assertTrue(json_response['non_field_errors'])
        self.assertFalse(json_response['form_errors'])

    def test_nearby_shops_latlon(self):
        form_data = {'latitude': 52.5565, 'longitude': 13.3911}
        encoded = urllib.urlencode(form_data)
        response = self.client.get(
            reverse('storelocator', args=['testlocator'])+'?'+encoded)
        json_response = json.loads(response.content)
        self.assertEqual(True, json_response['success'])
        self.assertEqual(200, response.status_code)
        self.assertTrue(json_response['shops'])
        self.assertFalse(json_response['non_field_errors'])
        self.assertFalse(json_response['form_errors'])

    def test_no_nearby_shops_latlon(self):
        form_data = {'latitude': 53.5565, 'longitude': 14.3911}
        encoded = urllib.urlencode(form_data)
        response = self.client.get(
            reverse('storelocator', args=['testlocator'])+'?'+encoded)
        json_response = json.loads(response.content)
        self.assertEqual(True, json_response['success'])
        self.assertEqual(200, response.status_code)
        self.assertEqual(0, len(json_response['shops']))
        self.assertFalse(json_response['non_field_errors'])
        self.assertFalse(json_response['form_errors'])

    def test_empty_querystring(self):
        form_data = {}
        encoded = urllib.urlencode(form_data)
        response = self.client.get(
            reverse('storelocator', args=['testlocator'])+'?'+encoded)
        json_response = json.loads(response.content)
        self.assertTrue(json_response['non_field_errors'])
        self.assertFalse(json_response['form_errors'])

    def test_jsonp_response(self):
        form_data = {'callback': 'jsonp'}
        encoded = urllib.urlencode(form_data)
        response = self.client.get(
            reverse('storelocator', args=['testlocator'])+'?'+encoded)
        self.assertIn("jsonp", response.content)

    def test_position_postalcode(self):
        """
        If both latlon and postalcode are passed as parameters, then latlon
        should have priority.
        """
        form_data = {
            'latitude': 52.5565, 'longitude': 13.3911, 'postalcode': 13359
        }
        encoded = urllib.urlencode(form_data)
        response = self.client.get(
            reverse('storelocator', args=['testlocator'])+'?'+encoded)
        json_response = json.loads(response.content)
        self.assertEqual(True, json_response['success'])
        self.assertEqual(200, response.status_code)
        self.assertTrue(json_response['shops'])
        self.assertFalse(json_response['non_field_errors'])
        self.assertFalse(json_response['form_errors'])

    def test_postalcode_distance(self):
        form_data = {'postalcode': '13359', 'iso': 'DE'}
        encoded = urllib.urlencode(form_data)
        response = self.client.get(
            reverse('storelocator', args=['testlocator'])+'?'+encoded)
        json_response = json.loads(response.content)
        self.assertFalse(json_response['shops'][0]['distance'])

    def test_latlon_distance(self):
        form_data = {'latitude': 52.5565, 'longitude': 13.3911}
        encoded = urllib.urlencode(form_data)
        response = self.client.get(
            reverse('storelocator', args=['testlocator'])+'?'+encoded)
        json_response = json.loads(response.content)
        self.assertTrue(json_response['shops'][0]['distance'])

    def test_num_queries(self):
        cache.clear()
        form_data = {'postalcode': '13359', 'iso': 'DE'}
        encoded = urllib.urlencode(form_data)

        with self.assertNumQueries(3):
            self.client.get(
                reverse('storelocator', args=['testlocator'])+'?'+encoded)

        with self.assertNumQueries(0):
            self.client.get(
                reverse('storelocator', args=['testlocator'])+'?'+encoded)



from storelocator.models import Location, StoreLocator, Shop

import factory


class LocationFactory(factory.DjangoModelFactory):

    FACTORY_FOR = Location

    iso = 'DE'
    postalcode = '10437'
    city = 'Berlin'
    state = 'Brandenburg'
    latitude = 52.5565
    longitude = 13.3911


class StoreLocatorFactory(factory.DjangoModelFactory):

    FACTORY_FOR = StoreLocator

    name = factory.Sequence(lambda i: 'StoreLocator-{0}'.format(i))
    slug = factory.Sequence(lambda i: 'storelocator-{0}'.format(i))


class ShopFactory(factory.DjangoModelFactory):

    FACTORY_FOR = Shop

    type = factory.Sequence(lambda i: 'type-{0}'.format(i))
    name = factory.Sequence(lambda i: 'shop-{0}'.format(i))
    city = 'Berlin'
    postalcode = '13359'
    street = 'Drontheimerstrasse 25'
    iso = 'DE'
    storelocator = factory.SubFactory(StoreLocatorFactory)


from django.db import models
from django.forms.models import model_to_dict
from django.utils.translation import ugettext_lazy as _

from geopy import distance


class Location(models.Model):

    iso = models.CharField(_('iso country code'), max_length=2)
    postalcode = models.CharField(_('postalcode'), max_length=20)
    city = models.CharField(_('place name'), max_length=180)
    state = models.CharField(_('state name'), max_length=100, blank=True)
    latitude = models.FloatField(_('latitude'))
    longitude = models.FloatField(_('longitude'))
    last_updated = models.DateTimeField(_('last updated'), auto_now_add=True)

    class Meta:
        verbose_name = _('location')
        verbose_name_plural = _('locations')
        ordering = ['iso', 'postalcode']

    def __unicode__(self):
        return u'%s - %s - %s' % (self.iso, self.city, self.postalcode)


class StoreLocator(models.Model):

    CACHE_KEY = 'storelocator_%s'

    name = models.CharField(_('name'), max_length=100)
    slug = models.CharField(_('slug'), max_length=55)

    class Meta:
        verbose_name = _('store locator')
        verbose_name_plural = _('store locators')

    def __unicode__(self):
        return u'%s - <%s>' % (self.name, self.slug)


class Shop(models.Model):

    type = models.CharField(_('type'), max_length=20)
    name = models.CharField(_('name'), max_length=140)
    city = models.CharField(_('place name'), max_length=180)
    postalcode = models.CharField(_('postalcode'), max_length=20)
    street = models.CharField(_('street'), max_length=140)
    iso = models.CharField(_('iso country code'), max_length=2)
    latitude = models.FloatField(_('latitude'), blank=True, null=True)
    longitude = models.FloatField(_('longitude'), blank=True, null=True)
    storelocator = models.ForeignKey(
        StoreLocator,
        verbose_name=_('store locator'),
        related_name='shops')

    class Meta:
        verbose_name = _('shop')
        verbose_name_plural = _('shops')

    def serialize(self):
        data = model_to_dict(self, fields=(
            'name', 'city', 'postalcode', 'street', 'latitude', 'longitude'))
        data['type'] = self.type.lower()

        # distance is set in the view if lat & lon are given.
        if hasattr(self, "distance"):
            data['distance'] = round(self.distance, 2)
        else:
            data['distance'] = False
        return data

    def distance_to(self, latitude, longitude):
        location = (self.latitude, self.longitude)
        return distance.distance((latitude, longitude), location)

    def __unicode__(self):
        return u'%s - %s (%s - %s)' % (
            self.type, self.name, self.latitude, self.longitude)



from django.conf.urls import patterns, url

from .views import StoreLocatorView


urlpatterns = patterns('storelocator.urls',
    url(r"^(?P<slug>[\w-]+)\.json$", StoreLocatorView.as_view(), name="storelocator"),
)